import paho.mqtt.client as mqtt
import sqlite3
import json
import base64
import socket
from datetime import datetime
import time

# MQTT configuration
BROKER_URL = "127.0.0.1"
BROKER_PORT = 1883
TOPIC = "application/+/device/+/event/up"
DB_PATH = "/usr/local/rs485/data.db"

# Function to check if the port is open
def is_port_open(host, port):
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        sock.settimeout(1)  # Set timeout to 1 second
        result = sock.connect_ex((host, port))
        return result == 0

# Wait for the port to open
while not is_port_open(BROKER_URL, BROKER_PORT):
    print(f"Waiting for port {BROKER_PORT} to open...")
    time.sleep(5)  # Wait 5 seconds before trying again

print(f"Port {BROKER_PORT} is open, proceeding to connect...")

# MQTT on_connect callback
def on_connect(client, userdata, flags, rc):
    print('Connected to MQTT broker')
    client.subscribe(TOPIC)
    print(f'Subscribed to topic: {TOPIC}')

# MQTT on_message callback
def on_message(client, userdata, msg):
    try:
        parsed_data = json.loads(msg.payload)
        # Adjust the format to parse the timezone correctly
        date = int(datetime.fromisoformat(parsed_data['time']).timestamp())
        date_h = format(date, 'x')
        deui = parsed_data['deviceInfo']['devEui']
        addr = parsed_data['devAddr']
        hexadecimal_data = base64.b64decode(parsed_data['data']).hex()
        fcnt = parsed_data['fCnt']
        fport = parsed_data['fPort']
        rssi = parsed_data['rxInfo'][0]['rssi']
        rssi_hex = format(abs(rssi), '02x')
        decode_data = json.dumps(parsed_data.get('object', {}))

        # SQLite operations
        with sqlite3.connect(DB_PATH) as conn:
            cursor = conn.cursor()

            # Create tables if not exist
            cursor.executescript(f"""
                CREATE TABLE IF NOT EXISTS listening (dev_eui TEXT PRIMARY KEY, fport TEXT);
                CREATE TABLE IF NOT EXISTS maindata (dev_eui TEXT PRIMARY KEY, dev_addr TEXT, fcnt INTEGER, payload TEXT, date TEXT, rssi TEXT);
                CREATE TABLE IF NOT EXISTS table{deui} (
                    id INTEGER PRIMARY KEY,
                    dev_eui TEXT,
                    dev_addr TEXT,
                    fcnt INTEGER,
                    rssi TEXT,
                    payload TEXT,
                    date TEXT
                );
                CREATE TABLE IF NOT EXISTS decode_data (dev_eui TEXT PRIMARY KEY, data TEXT, fport TEXT);
            """)

            cursor.execute("PRAGMA table_info(listening);")
            columns = [col[1] for col in cursor.fetchall()]
            if 'fport' not in columns:
                cursor.execute("ALTER TABLE listening ADD COLUMN fport TEXT;")

            # Check fport from listening table
            cursor.execute("SELECT fport FROM listening WHERE dev_eui = ?", (deui,))
            fport_query = cursor.fetchone()

            # Conditional check on fport
            if fport_query and fport_query[0] and fport_query[0] != str(fport):
                return

            # Insert into device-specific table
            cursor.execute(f"""
                INSERT INTO table{deui} (dev_eui, dev_addr, fcnt, rssi, payload, date)
                VALUES (?, ?, ?, ?, ?, ?)
            """, (deui, addr, fcnt, rssi_hex, hexadecimal_data, date_h))

            cursor.execute("""
                UPDATE maindata
                SET dev_addr = ?, fcnt = ?, payload = ?, date = ?, rssi = ?
                WHERE dev_eui = ?
            """, (addr, fcnt, hexadecimal_data, date_h, rssi_hex, deui))

            cursor.execute("""
                INSERT OR IGNORE INTO decode_data (dev_eui, data) VALUES (?, ?)
            """, (deui, decode_data))

            cursor.execute("""
                UPDATE decode_data SET data = ? WHERE dev_eui = ?
            """, (decode_data, deui))

    except Exception as e:
        print(f"Error processing message: {e}")

# MQTT setup
client = mqtt.Client(client_id='mqtt-subscriber' , clean_session=True)
client.on_connect = on_connect
client.on_message = on_message

# Connect to the broker
client.connect(BROKER_URL, BROKER_PORT)
client.loop_forever()

